import {
  Body,
  Config,
  Controller,
  Del,
  Get,
  Inject,
  Param,
  Post,
  Query,
} from '@midwayjs/decorator';
import {
  BaseController,
  FrameworkErrorEnum,
  ServiceError,
} from 'free-midway-component';
import { PersonalService } from '../../service/user/personal.service';
import {
  CDTO_personal,
  CVO_personal,
  FVO_personal,
  QDTO_personal,
  QVO_personal,
  UDTO_personal,
  UVAO_personal,
  LoginDTO,
  LoginVO,
  ResetPasswordDTO,
  FileDTO,
  ImportDTO,
} from '../../interface/user/personal.interface';
import {
  ApiOperation,
  ApiQuery,
  ApiResponse,
  ApiTags,
} from '@midwayjs/swagger';
import { Validate } from '@midwayjs/validate';
import { UserUtilService } from '../../service/util/user.util';
import { JwtService } from '@midwayjs/jwt';
// import get = require('lodash/get');
@ApiTags(['个人用户'])
@Controller('/personal')
export class PersonalController extends BaseController {
  @Inject()
  service: PersonalService;

  @Inject()
  userUtil: UserUtilService;

  @Inject()
  jwtService: JwtService;

  @Config('jwt.secret')
  jwtSecret;

  @Config('jwt.expiresIn')
  jwtExpiresIn;

  @Post('/login')
  @Validate()
  async login(@Body() body: LoginDTO) {
    const data = await this.service.findUserToLogin(body);
    let vo = new LoginVO(data);
    vo = JSON.parse(JSON.stringify(vo));
    const token = await this.jwtService.sign(vo, this.jwtSecret, {
      expiresIn: this.jwtExpiresIn,
    });
    return token;
  }

  @Post('/rp')
  @ApiOperation({ description: '修改密码' })
  async resetPassword(@Body() body: ResetPasswordDTO) {
    if (!body.id)
      throw new ServiceError('未找到用户信息', FrameworkErrorEnum.NOT_LOGIN);
    // 查询用户
    const user = await this.service.fetch(body.id);
    if (!user)
      throw new ServiceError('未找到用户', FrameworkErrorEnum.NOT_FOUND_DATA);
    // 修改密码
    await this.service.updateOne(body.id, { password: body.password });
    // 返回结果
    return 'ok';
  }

  @Post('/')
  @Validate()
  @ApiResponse({ type: CVO_personal })
  async create(@Body() data: CDTO_personal) {
    // 检查手机号
    // await this.userUtil.checkPhoneAndPid(data);
    const dbData = await this.service.create(data);
    const result = new CVO_personal(dbData);
    return result;
  }
  @Get('/')
  @ApiQuery({ name: 'query' })
  @ApiResponse({ type: QVO_personal })
  async query(
    @Query() filter: QDTO_personal,
    @Query('skip') skip: number,
    @Query('limit') limit: number
  ) {
    let query: any = {};
    if (filter.is_filter && filter.is_filter === 'true') {
      query = filter;
      delete query.is_filter;
    } else {
      if (this.ctx.user) {
        query = await this.service.dealQueryCondition(filter);
        delete query.is_filter;
      } else {
        query = filter;
        delete query.is_filter;
      }
    }
    const list = await this.service.query(query, {
      skip,
      limit,
      sort: { 'meta.createdAt': -1 },
    });
    const data = [];
    for (const i of list) {
      const newData = new QVO_personal(i);
      data.push(newData);
    }
    const total = await this.service.count(query);
    return { data, total };
  }

  @Get('/:id')
  @ApiResponse({ type: FVO_personal })
  async fetch(@Param('id') id: string) {
    const data = await this.service.fetch(id);
    const result = new FVO_personal(data);
    return result;
  }

  @Post('/:id')
  @Validate()
  @ApiResponse({ type: UVAO_personal })
  async update(@Param('id') id: string, @Body() body: UDTO_personal) {
    // 检查手机号和上级id
    // await this.userUtil.checkUpdateCardAndPid(id, body);
    const result = await this.service.updateOne(id, body);
    return result;
  }

  @Del('/:id')
  @Validate()
  async delete(@Param('id') id: string) {
    await this.service.delete(id);
    return 'ok';
  }

  @Post('/export')
  @Validate()
  async export(@Body() body: FileDTO) {
    const list = await this.service.exportMeta(body.file);
    const result = await this.service.export(list, body.filter);
    return result;
  }
  @Post('/import')
  @Validate()
  async import(@Body() body: ImportDTO) {
    const result = await this.service.import(body);
    return result;
  }
  async createMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }

  async updateMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }

  async deleteMany(...args: any[]) {
    throw new Error('Method not implemented.');
  }
}
