import { Provide } from '@midwayjs/decorator';
import { InjectEntityModel } from '@midwayjs/typegoose';
import { ReturnModelType } from '@typegoose/typegoose';
import {
  BaseService,
  FrameworkErrorEnum,
  ServiceError,
} from 'free-midway-component';
import { DockUser } from '../../entity/technology/dockUser.entity';
const assert = require('assert');
import _ = require('lodash');
import { Types } from 'mongoose';
const ObjectId = Types.ObjectId;
type modelType = ReturnModelType<typeof DockUser>;
@Provide()
export class DockUserService extends BaseService<modelType> {
  @InjectEntityModel(DockUser)
  model: modelType;

  /**
   * 根据展会id查询参展产品
   * @param {Object} query 查询条件
   * @param {Object} options 其他条件
   * @property {String} dock_id 展会id
   * @property {Object} condition 其他查询条件
   */
  async getProductListByDockId(condition) {
    const { dock_id, skip = 0, limit = 10, ...info } = condition;
    assert(dock_id, '缺少展会信息');
    // condition = this.dealFilter(condition);
    // 先查展会下的所有产品
    const pipeline: any = [{ $match: { dock_id } }];
    // 只留产品列表
    pipeline.push({ $project: { productList: 1 } });
    // 将产品列表数组拆开成单项Object
    pipeline.push({ $unwind: '$productList' });
    // 其他针对产品的查询条件
    // pipeline.push({ $match: info });
    // 满足条件的总数
    const totalRes = await this.model.aggregate([
      ...pipeline,
      { $group: { _id: null, total: { $sum: 1 } } },
    ]);
    let total = 0;
    if (totalRes.length > 0) {
      const totalHead = _.head(totalRes);
      total = _.get(totalHead, 'total');
    }

    // 分页的是产品,不是展会
    if (limit) {
      pipeline.push({ $skip: parseInt(skip) });
      pipeline.push({ $limit: parseInt(limit) });
    }
    let data = await this.model.aggregate(pipeline);
    data = data.map(i => {
      const { productList } = i;
      return productList;
    });
    data = _.compact(data);
    if (info.type) {
      data = data.filter(i => i.type === info.type);
      total = data.length;
    }
    if (info.field) {
      data = data.filter(i => i.field === info.field);
      total = data.length;
    }
    return { data, total };
  }
  async beforeCreate(data: {
    dock_id: string;
    user_id: string;
    productList: Array<any>;
  }) {
    const { dock_id, user_id } = data;
    let { productList } = data;
    const user = await this.model.findOne({ dock_id, user_id });
    if (!user) {
      const { productList = [] } = data;
      if (productList.length > 0) {
        data.productList = productList.map(i => {
          i.status = '0';
          i._id = new ObjectId();
          return i;
        });
      }
      return data;
    }
    productList = _.differenceBy(productList, user.productList, 'id');
    if (productList.length > 0) {
      productList = productList.map(i => {
        i.status = '0';
        i._id = new ObjectId();
        return i;
      });
    }
    user.productList = user.productList.concat(productList);
    return user;
  }

  async beforeUpdate(
    id: string,
    data: { productList: Array<any>; status: string }
  ) {
    const { productList = [] } = data;
    if (productList.length > 0) {
      data.productList = productList.map(i => {
        if (i.status === '1') data.status = '1';
        if (!i._id) i._id = new ObjectId();
        else i._id = new ObjectId(i._id);
        return i;
      });
    }
    return { _id: new ObjectId(id), data };
  }

  /**
   * 申请参展的产品审核
   * @param {Object} {id} 申请参展的产品的_id
   * @param {Object} {good_id, status} 要改变的状态
   * 只要有一个产品通过审核,该用户就更变为可通过状态=>need_pass_user
   */
  async goodCheck(id, { good_id, status }) {
    const object = await this.model.findOne({
      _id: new ObjectId(id),
      productList: { $elemMatch: { id: good_id } },
    });
    if (!object)
      throw new ServiceError(
        '未找到需要审核的产品信息',
        FrameworkErrorEnum.NOT_FOUND_DATA
      );
    const product = object.productList.find(f =>
      new ObjectId(good_id).equals(f.id)
    );
    if (!product)
      throw new ServiceError(
        '未在需要审核的产品中找到指定的产品',
        FrameworkErrorEnum.NOT_FOUND_DATA
      );
    product.status = status;
    await this.model.updateOne(
      { _id: new ObjectId(id), productList: { $elemMatch: { id: good_id } } },
      object
    );
    const need_pass_user = object.productList.some(e => e.status === '1');
    if (need_pass_user) this.userCheck({ id: object._id }, { status: '1' });
  }

  /**
   * 申请参展的用户审核
   * @param {Object} {id} 用户申请时生成的数据的id
   * @param {Object} {status} 要改变的状态
   */
  async userCheck(id, status) {
    const object = await this.model.findById(id);
    if (!object)
      throw new ServiceError(
        '未找到指定的用户申请',
        FrameworkErrorEnum.NOT_FOUND_DATA
      );
    object.status = status;
    await object.save();
  }
}
